/* 
Reziproke Frequenzmessung mit STM32G431 mit 8 - 11 stelliger Auflsung.
Eingangsfrequenbereich: 0,1 Hz - 150 MHz

Die Funktion der reziproken Frequenzmessung wird als bekannt vorausgesetzt.

Bei F1_messung() wird nicht die vorgegebene Messzeit abgewartet, um dann eine abschliessende
Auswertung durchzufhren, sondern es wird nach Mglichkeit jedes einzelne
Intervall erfasst. Dazu wird die Anforderung zur Erfassung der Intervalle 
per Timer7 ausgelst. Dieser setzt mit "F_SAMPLES" (200 kHz) den Ausgang "TRIGGER". 
Bei kleinerer Eingangsfrequenz wird jedes Intervall bewertet,
bei hheren Frequenzen soviele Perioden, wie in 5 s (synchronisiert) eintreffen.
Da Erfassung und Auswertung per ISR stattfinden, mu fr die restlichen 
Funktionen noch Rechenzeit brig bleiben. 
Derzeit werden ca. 70% fr die ISR von Timer17 verbraucht.

Bei der Erfassung eines Intervalls werden die Summen der Einzelwerte von Ereignissen
und Zeit (reg_xi, reg_yi), die Summe aus diesen Einzelsummen (reg_sum_xi, reg_sum_yi)
und die Summen der Produkte aus reg_xi*reg_xi und reg_xi*reg_yi (reg_sum_xi2, reg_sum_xiyi)
gebildet. Ferner wird die Anzahl der Ereignisse (reg_n) gezhlt.
Nach Ablauf der Messzeit, kann aus diesen Werten mit linearer Regression die
verfeinerte Steigung der Geraden errechnet werden.

Die Grundauflsung ist 8-stellig, steigt jedoch mit Wurzel aus reg_n mit zunehmender
Eingangsfrequenz. 
Bei 1 s Messzeit sind folgende Auflsungen zu erwarten:

0,1 Hz - 50 Hz:    8-stellig
50 Hz - 5 kHz:     9-stellig
>=5 kHz:          10-stellig

Erhht man die Messzeit auf 10 s erhlt man eine weitere Stelle Auflsung hinzu.
Zu beachten ist, da fr derartige Ergebnisse uerst stabile Signale fr
Eingangs- und Referenzfrequenz bentigt werden.

Weiterer Messkanal:
Zustzlich wird eine weitere Frequenz "F2_ergebnis" per Timer8 gemessen. Der Messbereich
ist 0,1 Hz - ca. 1 MHz. Hauptaufgabe dieser Messung ist, den verwendeten Referenztakt
fr F1 anhand eines 1pps GPS-Signal zu messen und die Auswertung entsprechend
zu korrigieren. Dies geschieht mit der Funktion "bewerte_gps_signal()" aber nur unter
der Voraussetzung, dass der verwendete Takt max. +/- 5 ppm vom 1 pps-Signal abweicht.
Der ermittelte Korrekturwert wird zyklisch im EEPROM abgelegt und beim nchsten
Einschalten verwendet.
Mit "F2_5kHz_SPERRE" wird die max. Eingangsfrequenz von F2 auf 5 kHz begrenzt. Damit
wird der Controller nicht mit permanenten ISR-Aufrufen (F2_CAPT) blockiert.

2021-05-13: V1.4 Automatische Umschaltung der Referenzfrequenz
Fr die F1-Messung kann fr die Auflsung zwischen den vier Modi "normal", "hoch 166.7",
"hoch 170.0", "hoch 171.7" und "auto" gewhlt werden.

normal:     8-stell. Ergebnisse ohne Regressionsbewertung
hoch: 166.7 hohe Auflsung mit fester Referenzfrequenz von 166,7... MHz @ 10 MHz
hoch: 170.0 hohe Auflsung mit fester Referenzfrequenz von 170.0... MHz @ 10 MHz
hoch: 171.7 hohe Auflsung mit fester Referenzfrequenz von 171,7... MHz @ 10 MHz
hoch: auto  automatische Umschaltung der Referenzfrequenz zur Vermeidung geradzahliger,
            phasenstarrer Teilerverhltnisse Fref/Fin bei Fin >= 10 kHz.
            Das ergibt die beste Auflsung bei variabler Eingangsfrequenz.

2021-03-16
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#define F1_MESSUNG

#include "system_G431.h"


#define F2_5kHz_SPERRE                // schtzt F2 vor ISR-Blockade

#define TEMP_STR_LEN      50          // fuer Ergebnis String
#define MAX_ERGEBNIS_STR  20          // Ausgabelnge begrenzen

#define F_SAMPLES         200000      // max. Messrate festlegen
#define KONTROLL_AUSGANG  2           // PF1 fr Laufzeitmessung

double rundung[] = {5e0, 5e-1,5e-2,5e-3,5e-4,5e-5,
                    5e-6,5e-7,5e-8,5e-9,5e-10,5e-11,
                    5e-12,5e-13,5e-14,5e-15};

char f_dim[][10][6]={{" GHz"," MHz"," kHz"," Hz "," mHz"," sek"," ms "," us "," ns "," ps "},
                    {"E+9","E+6","E+3","E+0","E-3","E+0","E-3","E-6","E-9","E-12"}};  // Dimensionen zum Messwert
char u_dim[][5][6]={{" Grpm"," Mrpm"," krpm"," rpm"," mrpm"},
                    {"E+9","E+6","E+3","E+0","E-3"}};

enum ablauf {MESSEN=0, AUSLESEN, AUSWERTEN, FERTIG};        // die Phasen der Messung
enum anzeige {WANDEL_F=0, WANDEL_P, WANDEL_D};

uint16_t    led_ein_zeit = DEF_LED_EIN_ZEIT;
uint8_t     gps_aktiv;
uint16_t    gps_fil_cnt,          // T3-Filter, Anzahl Werte
            gps_fil_laenge = DEF_GPS_FILTER,
            ee_schreib_intervall, // gleiches intervall wie gps_fil_laenge
            gps_fil_index,        // Filter, Index auf Einzelwerte
            gps_totzeit;          // Einschaltverzoegerung abwarten

double      gps_fil[MAX_GPS_FILTER], // letzte Einzelwerte
            gps_fil_sum,
            ref_frequenz;         // akt. Summe der Einzelwerte
 
int32_t     frequenz_offset;

uint8_t     F1_vorteiler_aktiv,
            F2_vorteiler_aktiv;
uint8_t     ausgabe_sperre, update_header;

// Variablen zum Umtasten der Referenzfrequenz
 double       F1_ref_low,       // untere F_ref
              F1_ref_mid,       // mittlere
              F1_ref_high,      // obere
              F_temp,
              F1_ref_akt;       // aktuell verwendet
 uint32_t     F1_rest_low,      // Teilerrest bei unterer F_ref
              F1_rest_mid,      // mittlerer
              F1_rest_high,     // und oberer
              F1_pll_m,         // aktueller Wert um F_ref_low einzustellen
              F1_pll_n_low,     // und fr N
              F1_pll_n_mid,     // und fr N
              F1_pll_n_high,    // und fr N
              F1_pll_n_temp,
              F1_opt_hys,       // 20% der Sollfrequenz
              F1_pll_alt,       // letzte Bewertung verzgert anzeigen
              F1_pll_status;
uint32_t      capt_err_cnt;

char p_str[TEMP_STR_LEN];

volatile double     F1_taktfrequenz, 
                    F2_taktfrequenz;                // hier nicht verwendet

double              F1_ergebnis,                    // in Hz
                    eff_zeit1;                      // eff. Messzeit in s
double              F1_ref_frequenz = STD_TAKTFREQUENZ;  // Vorgabewerte
int32_t             F1_ref_offset = 0;              // fr Abgleich von F1_ref
uint32_t            F1_messzeit = DEF_F1_MESSZEIT,
                    F1_timeout = DEF_TIMEOUT,
                    F1_stellen = DEF_STELLEN,
                    F1_faktor  = DEF_FAKTOR,
                    F1_upm_divisor  = DEF_UPM_DIVISOR;
                    
volatile uint8_t    F1_messwert_vorhanden,          // gueltige Messungen
                    F1_mess_status;                 // Ablaufsteuerung
                                   
volatile uint32_t   F1_mess_dauer;                  // in ms
uint32_t            F1_start_zeit,                  // rel. Zeit: Beginn der Messung
                    F1_zeit_ueberlauf,
                    F1_end_zeit,                    // Zeitpunkt der Auswertung
                    F1_mess_zeit,                   // genaue Zeit der Messung
                    F1_ueberlauf; 
uint32_t   F1_mess_ereignisse,             // Impulse der Messung
                    F1_start_ereignis,              // Impulse zu Beginn der Messung
                    F1_end_ereignis;                // Impulse am Ende der Messung



double              F2_ergebnis,                    // in Hz
                    eff_zeit2;                      // eff. Messzeit in s
double              F2_ref_frequenz = STD_TAKTFREQUENZ;  // Vorgabewerte
int32_t             F2_ref_offset = 0;              // fr Abgleich von F2_ref
uint32_t            F2_messzeit = DEF_F2_MESSZEIT,
                    F2_timeout = DEF_TIMEOUT,
                    F2_stellen = DEF_STELLEN,
                    F2_faktor  = DEF_FAKTOR,
                    F2_upm_divisor  = DEF_UPM_DIVISOR;

volatile uint8_t    F2_messwert_vorhanden,          // gueltige Messungen
                    F2_mess_status;                 // Ablaufsteuerung
                                   
volatile uint32_t   F2_mess_dauer;                  // in ms

uint32_t            F2_start_ereignis,              // Impulse zu Beginn der Messung
                    F2_start_zeit,                  // rel. Zeit: Beginn der Messung
                    F2_zeit_ueberlauf,
                    F2_end_zeit,                    // Zeitpunkt der Auswertung
                    F2_mess_zeit,                   // genaue Zeit der Messung
                    F2_mess_ereignisse,             // Impulse der Messung
                    F2_anzahl; 
volatile uint32_t   F2_end_ereignis;                // Impulse am Ende der Messung


// Variablen fr Regressionsberechnung
uint32_t  reg_n;                          // Anzahl Ereignisse bei Regression
volatile  uint32_t reg_ereignisse;        // und Endwert zur weiteren Auswertung
uint64_t  reg_xi, reg_yi;                 // summe fortlaufender Einzelwerte       
uint64_t  reg_sum_xi, reg_sum_yi;         // Summe aller Intervalle

double    reg_sum_xiyi,                   // Summe xi * yi
          reg_sum_xi2;                    // Summe xi
double    reg_xq, reg_yq;                 // Mittelwerte xi/n und yi/n

uint8_t berechne_stellen1(double zeit, uint32_t ereignisse)
{
static double offset;
uint8_t stellen = MIN_STELLEN;	          // minimal
  if(ereignisse > 50) zeit *= 10;         // bei Regression mehr Stellen zulassen
  if(ereignisse > 5000) zeit *= 10;       // noch eine weitere
  if(zeit+offset >= 990.0) {
    stellen = 11;
    offset = 100.0;
  } else if(zeit+offset >= 99.0) {
    stellen = 10;
    offset = 10.0;
  } else if(zeit+offset >= 9.9) {         // Anzahl der gltigen Stellen
    stellen = 9;                          // verringert sich bei krzeren Messzeiten
    offset = 1.0;
  } else if(zeit+offset >= 9.9e-1) {
    stellen = 8;
    offset = 0.1;
  } else if(zeit+offset >= 9.9e-2) {
    stellen = 7;
    offset = 0.01;
  } else if(zeit+offset >= 9.9e-3) {
    stellen = 6;
    offset = 0.001;
  } else if(zeit+offset >= 9.9e-4) {
    stellen = 5;
    offset = 0.0001;
  } else offset = 0;
  if(stellen < F1_MIN_STELLEN) stellen = F1_MIN_STELLEN;
  return(stellen);
}

uint8_t berechne_stellen2(double zeit)
{
static double offset;
uint8_t stellen = MIN_STELLEN;	// minimal
  if(zeit+offset >= 99.0) {
    stellen = 10;
    offset = 10.0;
  } else if(zeit+offset >= 9.9) {
    stellen = 9;
    offset = 1.0;
  } else if(zeit+offset >= 9.9e-1) {
    stellen = 8;
    offset = 0.1;
  } else if(zeit+offset >= 9.9e-2) {
    stellen = 7;
    offset = 0.01;
  } else if(zeit+offset >= 9.9e-3) {
    stellen = 6;
    offset = 0.001;
  } else if(zeit+offset >= 9.9e-4) {
    stellen = 5;
    offset = 0.0001;
  } else offset = 0;
//  if(stellen < min_stellen) stellen = min_stellen;
  return(stellen);
}


void messwert_lcd1(char *s)
{
  if(!ausgabe_sperre) lcd_messwert_zeile1(s);
}  

void messwert_lcd2(char *s)
{
  if(!ausgabe_sperre) lcd_messwert_zeile2(s);
}  


// Routine zur Wandlung und Ausgabe eines Mewertes
void wandel_Fx(char *header, double x, char zeige_periode, char *s, uint8_t stellen)    // Anzeige von Frequenz oder Periode
{
  
int i,j,dez_punkt,dimension;
char *p;

  if(zeige_periode == WANDEL_F && nur_MHz)
  {
    if(x >= 1e6) stellen--;
    if(x >= 1e7) stellen--;
    if(x >= 1e8) stellen--;
    if(x >= 1e9) stellen--;
    sprintf(s,"%.*f MHz",stellen,x*1e-6);
    if(*s == '0') *s = ' ';
    return;
  }    

  dez_punkt = 0;
  dimension = 3;
  p = header;
  while(*p) *s++ = *p++;                    // "P = " fr Periodendauer
  if(x >= 0.001) {                          // 1mHz ist Untergrenze
    if(zeige_periode == WANDEL_P) {
      x = 1/x;                              // Kehrwert bilden
      dimension=5;                          // und in Sekunden als Dimension
    }
    while(x<1.0) {x*=1000.0;dimension++;}   // in den Hz-Bereich bringen
    while(x>=1000.0) {x*=0.001;dimension--;}
    while(x >= 10.0) {
      x *= 0.1;
      dez_punkt++;
    }
    x += rundung[stellen];                  // runden
    if(x >= 10.0) {			                    // Ueberlauf bei Rundung
      x *= 0.1;                             // korrigieren
      dez_punkt++;
      if(dez_punkt > 2) {		                // Ueberlauf der Dimension
        dimension--;
        dez_punkt = 0;
      }
    }
  } else {
    x = 0.0; dez_punkt=0; dimension=3;      // 0.00000 Hz ausgeben
  }
  
  for(i=0;i<stellen;i++) {
    j = (uint8_t)x;
    *s++ = (j+'0');
    if(i == dez_punkt) {
      if(anzeige_format & 2)
        *s++ = (',');
      else
        *s++ = ('.');
    }  
    x -= j;
    x *= 10;
  }

  if(zeige_periode == WANDEL_D)
    p = u_dim[anzeige_format&1][dimension];
  else
    p = f_dim[anzeige_format&1][dimension];
  while(*p) *s++ = *p++;                    // Dimension anhngen
  *s++ = 0;                                 // string abschliessen
  *s = 0;                                   // string abschliessen
  s[MAX_ERGEBNIS_STR] = 0;                  // zur Sicherheit begrenzen
}

void Fx_str_mittig(char *header, double x, char zeige_periode, char *s, uint8_t stellen)    // Anzeige von Frequenz oder Periode
{
char p[50];
int i,j;
  *p = 0;
  for(i = 0; i < MAX_ERGEBNIS_STR; i++)     // Ergebnisstring lschen
    s[i] = ' ';
  s[MAX_ERGEBNIS_STR] = 0;                  // Ende von s
  
  wandel_Fx(header,x,zeige_periode,p,stellen);// wandeln
  i = strlen(p);
  i = lcd_breite - i;                       // auf 16 Zeichen/Zeile einmitten
  if(i < 0) i = 0;                          // Unterlauf verhindern
  j = i/2;
  i = 0;
  while(p[i]) s[j++] = p[i++];              // in den Ergebnisstring schreiben
  s[lcd_breite] = 0;                        // Lnge sicher begrenzen
} 

void loesche_regression(void)
{
  reg_n = 0;                                  // und alle Werte der Regressionsberechnung
  reg_xi = reg_yi = 0;                        // lschen
  reg_sum_xi = reg_sum_yi = 0;
  reg_sum_xiyi = reg_sum_xi2 = 0;
  reg_xq = reg_yq = 0;
}

// 1 kHz ISR mit T6 erzeugen
void init_t6(void)
{
static uint32_t alter_takt;
  if(alter_takt != SystemCoreClock) {
    RCC->APB1ENR1 |= RCC_APB1ENR1_TIM6EN;
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOBEN;      // PortB aktivieren
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOFEN;      // PortF aktivieren
    alter_takt = SystemCoreClock;
    TIM6->CR1 = 0;                            // stoppen
    GPIOB->MODER &= ~(CLR_MODER << (LED_FERTIG * 2));// MODER PB4 = 0
    GPIOB->MODER |= (OUT << (LED_FERTIG * 2));// PB4 auf Ausgang         
    GPIOF->MODER &= ~(CLR_MODER << (LED_1PPS * 2));// MODER PF1 = 0
    GPIOF->MODER |= (OUT << (LED_1PPS * 2));  // PF1 auf Ausgang         

    TIM6->ARR = 1000-1;                       // Teiler fr 1 kHz
    TIM6->PSC = SystemCoreClock/MHZ_FAKTOR-1; // Vorteiler auf 1 MHz
    TIM6->DIER = TIM_DIER_UIE;                // und Interrupt freigeben
    
    NVIC_EnableIRQ(TIM6_DAC_IRQn);
    NVIC_SetPriority(TIM6_DAC_IRQn,PRIO_TIM6);// geringe Prioritaet
  }    
  TIM6->CR1 = TIM_CR1_CEN | TIM_CR1_ARPE;     // starten ARR fr nderungen puffern
} 

// Messdauer im 1 ms Raster inkrementieren
void TIM6_DAC_IRQHandler(void)                // Aufruf mit 1 kHz
{
  TIM6->SR = 0;                               // flag loeschen
  F1_mess_dauer++;                            // Mesdauer in ms-Raster
  F2_mess_dauer++;                            // Mesdauer in ms-Raster
  __ISB();                                    // gelschte Flags abwarten
} 

// 200 kHz ISR mit T7 erzeugen
void init_t7(void)
{
static uint32_t alter_takt;
  if(alter_takt != SystemCoreClock) {
    RCC->APB1ENR1 |= RCC_APB1ENR1_TIM7EN;
    alter_takt = SystemCoreClock;
    TIM7->CR1 = 0;                            // stoppen
    TIM7->ARR = SystemCoreClock/F_SAMPLES-1;  // max. Abfragefrequenz
    TIM7->DIER = TIM_DIER_UIE;                // und Interrupt freigeben
    
    NVIC_EnableIRQ(TIM7_IRQn);
    NVIC_SetPriority(TIM7_IRQn,PRIO_TIM7);    // geringe Prioritaet
    
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOAEN;      // PortA aktivieren
    GPIOA->MODER = (GPIOA->MODER &~(CLR_MODER << F1_TRIGGER*2)) 
                    | (OUT << F1_TRIGGER*2);         // als Trigger-Ausgang
    GPIOA->OSPEEDR  |= V_HIGH_SPEED << F1_TRIGGER*2; // mit steilen Flanken
    set_pin(GPIOA, F1_TRIGGER,0);             // alte Anforderung lschen, TIM7 setzt neue Anforderung 
  }
  TIM7->CR1 = TIM_CR1_CEN | TIM_CR1_ARPE;     // starten ARR fr nderungen puffern
} 

void TIM7_IRQHandler(void)                    // Aufruf mit F_SAMPLES
{
    TIM7->SR = 0;
    set_pin(GPIOA, F1_TRIGGER, 1);            // neue Werte anfordern
  __ISB();                                    // gelschte Flags abwarten
}

void init_t2(void)
{
uint32_t temp;  
  RCC->APB1ENR1 |= RCC_APB1ENR1_TIM2EN;
	RCC->AHB2ENR |= RCC_AHB2ENR_GPIOAEN;                // PortA aktivieren
  TIM2->CR1 = 0;                                      // T2 stoppen, falls schon aktiv
  
// alternative Funktion der Zaehlereingaenge einschalten 
  temp = GPIOA->MODER;
  temp &= ~((CLR_MODER << (F1_COUNT * 2) |
             CLR_MODER << (F1_CAPT * 2)));                       
  temp |= (AF << (F1_COUNT * 2) |
           AF << (F1_CAPT * 2));
  GPIOA->MODER = temp;
  
// Pullup zu den Eingaengen einschalten
  GPIOA->PUPDR |= (PULL_UP << (F1_COUNT * 2) |
                   PULL_UP << (F1_CAPT * 2));                       

// PA0 als Zhlereingang, PA1 als capture-input zu T2
  GPIOA->AFR[0] |=  (AFR_TIM2 << (F1_COUNT * 4) | 
                     AFR_TIM2 << (F1_CAPT * 4));                   
  temp = GPIOB->MODER;
  temp &= ~(CLR_MODER << (Q0_EINGANG * 2)) |
           (IN << (Q0_EINGANG * 2));                  // als Eingang fr LSB Ereignisse
  GPIOB->MODER = temp;

  TIM2->CCMR1 = TIM_CCMR1_CC2S_0;                     // capture zuordnung  IC2 -> TI2
  TIM2->CCER =  TIM_CCER_CC2E;                        // pos. Flanken an IC2 freigeben
  TIM2->SMCR |= TIM_SMCR_TS_2 + TIM_SMCR_TS_0;        // ext. Signal von TI1
  TIM2->SMCR |= TIM_SMCR_SMS_2 + TIM_SMCR_SMS_1 + TIM_SMCR_SMS_0;   // Zhlereingang von TI1
  TIM2->CR1 |=  TIM_CR1_CEN;                          // T2 starten  
}

// Timer17 liefert die Zeit per Capture-Eingang PB5
void init_t17(void)
{
uint32_t temp;  
  RCC->APB2ENR |= RCC_APB2ENR_TIM17EN;                // Timer17 aktivieren fr HSE-Messung
	RCC->AHB2ENR |= RCC_AHB2ENR_GPIOBEN;                // PortB aktivieren
  TIM17->CR1 = 0;                                     // T17 stoppen, falls schon aktiv

  temp = GPIOB->MODER;
  temp &= ~(CLR_MODER << (F1_TIME_CAPT * 2)) |
           (AF << (F1_TIME_CAPT * 2));                // alternative Funktion
  GPIOB->MODER = temp;
  GPIOB->PUPDR |= (PULL_UP << (F1_TIME_CAPT * 2));    // Pullup einschalten

// PB5 als capture-input zu T17
  GPIOB->AFR[0] |=  (AFR_TIM17 << ((F1_TIME_CAPT) * 4));                   


  NVIC_EnableIRQ(TIM1_TRG_COM_TIM17_IRQn);            // Int-T17 aktivieren
  NVIC_SetPriority(TIM1_TRG_COM_TIM17_IRQn,PRIO_TIM17);
  TIM17->CCMR1 = TIM_CCMR1_CC1S_0;                    // capture zuordnung  IC2 -> TI2
  TIM17->CCER =  TIM_CCER_CC1E;                       // pos. Flanken an IC2 freigeben
 
  TIM17->DIER |= TIM_DIER_CC1IE +TIM_DIER_UIE;        // mit interrupts
  TIM17->CR1 |= TIM_CR1_CEN;                          // T17 starten
}


// ISR zu Timer17
// synchron zu einem Eingangsimpuls Ereignisse und Zeitpunkt ermitteln
// anschlieend Auswertung mit lin. Regression
// ferner berlaufe bei der Zeitmessung zhlen
void TIM1_TRG_COM_TIM17_IRQHandler(void)
{
uint16_t temp;
static double reg_divisor;
  if(TIM17->SR & TIM_SR_CC1IF) {                // bei Capture-Ereignis
    temp = TIM17->CCR1;                         // Capture-Wert Zeit
    F1_end_zeit = F1_zeit_ueberlauf + temp;     // Endwert Zeit
    if((TIM17->SR & TIM_SR_UIF) && (temp < 0x8000)) {   // evtl. Ueberlauf T17 noch offen?
      F1_end_zeit += 0x10000;                   // dann korrigieren
    } 
// Input-Capture testen, ob neuer Wert vorliegt    
    if(!(TIM2->SR & TIM_SR_CC2IF)) {  
      capt_err_cnt++;                           // merken und Intervall vergrern
    } else {                                    // wenn alles in Ordnung
      F1_end_ereignis = TIM2->CCR2;             // 32 Bit Capture-Wert reicht aus!
#ifdef MIT_VORTEILER_2zu1
      F1_end_ereignis <<= 1;                    // Platz fr LSB machen
      if(!(GPIOB->IDR & BIT(Q0_EINGANG)))       // LSB invertiert auslesen
        F1_end_ereignis++;                      // und addieren
#endif    

// jedes Intervall berechnen und weiterverarbeiten
      F1_mess_zeit = F1_end_zeit;               // Zeit-Differenz bilden
      F1_mess_zeit -= F1_start_zeit;            // Zeit-Differenz bilden
      F1_start_zeit = F1_end_zeit;
      F1_mess_ereignisse = F1_end_ereignis;     // Impuls-Differenz
      F1_mess_ereignisse -= F1_start_ereignis;  // Impuls-Differenz
      F1_start_ereignis = F1_end_ereignis;      // fuers naechste Intervall

// Anzahl und Einzelwerte fr die Regression verarbeiten      
      reg_n++;                          // anzahl der Einzelwerte
      reg_xi += F1_mess_ereignisse;     // Einzelwerte addieren
      reg_yi += F1_mess_zeit;           // dto
      reg_sum_xi += reg_xi;             // Summe aus Summe der Einzelwerte
      reg_sum_yi += reg_yi;             // dto
      reg_sum_xiyi += reg_xi*reg_yi;    // Summe der Produkte
      reg_sum_xi2  += reg_xi*reg_xi;    // Summe der Quadrate
    
// Auswertung nach Ablauf der Messdauer starten    
      if(F1_mess_status == MESSEN &&            // wenn Auswertung aktiv
        F1_mess_dauer >= F1_messzeit) {        // und Mindestzeit erreicht
        F1_ref_frequenz = F1_taktfrequenz + F1_ref_offset;  // immer aktuellen Wert verwenden
        reg_xq = (double)reg_sum_xi/(double)reg_n;
        reg_yq = (double)reg_sum_yi/(double)reg_n;
        reg_divisor = (reg_sum_xi2 - reg_n * (reg_xq * reg_xq));
        if((F1_reg_modus != NORMAL) && reg_n >= 2 && reg_divisor != 0.0) {  // Regression erst ab zwei Werten          
          F1_ergebnis = F1_ref_frequenz / ((reg_sum_xiyi - reg_n * (reg_xq * reg_yq)) / reg_divisor);
        } else {                                // wenn nur ein Ereignis in reg_xi
          F1_ergebnis = ((double)reg_xi/(double)reg_yi)*F1_ref_frequenz;  // Berechnung ohne Regression
        }  
        F1_mess_status = FERTIG;                // Daten fertig fuer Anzeige
// Endwerte fr Debugger      
        reg_ereignisse = reg_n;
        eff_zeit1 = reg_yi/F1_ref_frequenz;     // eff. Messzeit in s
        F1_mess_dauer = 0;                      // neue Messzeit starten
        loesche_regression();
      }
    }  
    set_pin(GPIOA, F1_TRIGGER,0);             // alte Anforderung lschen, TIM7 setzt neue Anforderung 
  }

// ganz zum Schluss noch die berlufe bercksichtigen
  if(TIM17->SR & TIM_SR_UIF) {                // bei berlufen von T17
     TIM17->SR = ~TIM_SR_UIF;                 // flag wieder loeschen
     F1_zeit_ueberlauf += 0x10000;            // und berlufe zhlen
  }
  __ISB();                                    // gelschte Flags abwarten
}


// Timer 8 fr Ereignis und Zeitmessung F2
void init_t8(void)
{
uint32_t tmp;  
  RCC->APB2ENR |= RCC_APB2ENR_TIM8EN;
	RCC->AHB2ENR |= RCC_AHB2ENR_GPIOAEN;            // PortA aktivieren
  NVIC_EnableIRQ(TIM8_CC_IRQn);                   // Int-T8 Input-Capture aktivieren
  NVIC_SetPriority(TIM8_CC_IRQn,PRIO_TIM8);

  TIM8->CR1 = 0;                                  // T8 stoppen, falls schon aktiv
  tmp = GPIOA->MODER;
  tmp &= ~(CLR_MODER << (F2_CAPT * 2));           // MODER auf 0                
  tmp |= (uint32_t)(AF << (F2_CAPT * 2));         // altern. Funktion
  GPIOA->MODER = tmp;

// PA15 als capture-input zu T8
  GPIOA->AFR[1] |= (AFR_TIM8 << (F2_CAPT-8) * 4);                   
  TIM8->CCMR1 =  TIM_CCMR1_CC1S_0;                // capture zuordnung  IC1 -> TI1
  TIM8->CCER = TIM_CCER_CC1E;                     // pos. Flanken an IC1 freigeben

  TIM8->DIER |= TIM_DIER_CC1IE + TIM_DIER_CC4IE;    // mit Capture + berlauf interrupts
  TIM8->CR1 |= TIM_CR1_CEN;                       // T8 starten
}


// Capture ISR zu Timer8
void TIM8_CC_IRQHandler(void)
{
  uint32_t temp;
  if(TIM8->SR & TIM_SR_CC1IF) {                   // bei Capture-Ereignis
    temp = TIM8->CCR1;                            // lscht CC1IF
#ifdef F2_5kHz_SPERRE    
    TIM8->DIER &= ~TIM_DIER_CC1IE;                // weiter F2_CAPT sperren
#endif    
    F2_anzahl++;
    if(F2_mess_status == AUSLESEN) {
      F2_end_ereignis = F2_anzahl;
      F2_end_zeit = F2_zeit_ueberlauf + temp;     // Endwert Zeit
      if((TIM8->SR & TIM_SR_CC4IF) && (temp < 0x8000)) { // evtl. Ueberlauf T8 noch offen?
         F2_end_zeit += 0x10000;                  // dann korrigieren
      }  
      F2_mess_status = AUSWERTEN;                 // Daten fertig fuer Auswertung
    }
  }
  if(TIM8->SR & TIM_SR_CC4IF) {                   // bei berlufen von T8
    TIM8->SR = ~TIM_SR_CC4IF;                     // flag wieder loeschen
    F2_zeit_ueberlauf += 0x10000;                 // und berlufe zhlen
#ifdef F2_5kHz_SPERRE 
    TIM8->DIER |= TIM_DIER_CC1IE;                 // F2_CAPT wieder freigeben
#endif    
  }
  __ISB();                                        // gelschte Flags abwarten
}

void loesche_gps_filter()
{
  for(gps_fil_index = 0; gps_fil_index < gps_fil_laenge; gps_fil_index++)
    gps_fil[gps_fil_index] = 0l;
  gps_fil_index = 0;
  gps_fil_cnt = 0;
  gps_fil_sum = 0.0;
  gps_led_cnt = 0;                          // GPS-Signal fehlt: LED ausschalten
  ee_schreib_intervall = gps_fil_laenge;    // neu warten
  gps_totzeit = GPS_TOTZEIT;
}

void bewerte_gps_signal(double frequenz) {
uint16_t  led_temp; 

  frequenz = frequenz / F_ref;              // normieren
  gps_fil_sum -= gps_fil[gps_fil_index];    // letzen Wert abziehen
  gps_fil_sum += frequenz;                  // neuen Wert addieren
  gps_fil[gps_fil_index++] = frequenz;      // und ablegen
  if(gps_fil_index >= gps_fil_laenge)
    gps_fil_index = 0;
  gps_fil_cnt++;                            // mitzaehlen, ob Filter 'gefuellt'
  if(gps_fil_cnt >= gps_fil_laenge) {       // Filter ist 'eingeschwungen'
    gps_fil_cnt = gps_fil_laenge;           // nicht weiter zaehlen
    ref_frequenz = (gps_fil_sum / gps_fil_laenge) * F_ref; // wieder skalieren
    gps_led_cnt = GPS_TIMEOUT;              // Dauerlicht durch Nachtriggern
    F1_taktfrequenz = ref_frequenz;         // jede Sekunde neu anpassen
    ee_schreib_intervall++;
    if(ee_schreib_intervall >= gps_fil_laenge) {
      ee_schreib_intervall = 0;             // neu zaehlen
      frequenz_offset = (int)((ref_frequenz - F_ref) /  // auf 0,1 ppb/Schritt
                              F_OFFSET_SCHRITT / F_ref);// skalieren
      speicher_akt_offset(frequenz_offset); // zyklisch letzten Wert speichern                                      
    }
  } else {
    if(gps_fil_laenge >= GPS_LED_STUFEN) {  // Abstufung, um zu kurze Zeiten zu verhindern
      led_temp = (gps_fil_cnt/(gps_fil_laenge/GPS_LED_STUFEN))*(gps_fil_laenge/GPS_LED_STUFEN);
      if(led_temp <= (gps_fil_laenge/GPS_LED_STUFEN)/2) 
        led_temp = (gps_fil_laenge/GPS_LED_STUFEN)/2+1;  // minimal 1
    } else led_temp = gps_fil_cnt;                  // Dauerlicht, wenn Filter gefllt ist
    gps_led_cnt = (1000*led_temp)/gps_fil_laenge;   // Blinkzeit in ms, wenn Filter noch nicht voll ist
  }
}

/* direkte Messung mit internen Timern mit max. 8 Stellen/s
Timer8 zhlt die externen Impulse per Capture-Signal
und mit die Zeit mit internem Takt
*/
void F2_messung(uint8_t init_timer)
{
static uint8_t  init = 0, gps_laeuft = 0;
uint32_t eff_stellen;
static char s[TEMP_STR_LEN];
  if(init_timer) {
    init = 0;                                   // Anzeige + Messung initialisieren
    init_t6();                                  // T6 fr Messdauer in 1 ms Schritten
    init_t8();                                  // T8 als Ereignisszhler mit capture-modus starten
  }
    if(!init) {
      F2_mess_status  = MESSEN;                 // mit MESSEN Beginnen
      F2_mess_dauer =  F2_messzeit;             // und 1. Messung sofort abschlieen
      F2_messwert_vorhanden  = 0;               // dabei 1. Ergebnis verwerfen
      lcd_zeile3("  1-pps Signal  "); 
      lcd_zeile4("    Signal?     ");
      init = 1;
    } 
  
    if(F2_mess_dauer >= F2_timeout) {           // bei fehlenden Eingangsimpulsen
      F2_mess_status = MESSEN;                  // Abbruch und Neustart
      init = 0;                                 // neu initialisieren
    }  

    if(F2_mess_status == MESSEN  && F2_mess_dauer >= F2_messzeit) { // nach Mindestmesszeit
      F2_mess_status = AUSLESEN;                // Zhler bei nchstem Eingangsimpuls auslesen
    }
  
    if(F2_mess_status == AUSWERTEN) {           // Ereignisse und Zeitpunkt ermittelt
      F2_mess_zeit = F2_end_zeit - F2_start_zeit; // Zeit-Differenz bilden
      F2_start_zeit = F2_end_zeit;              // fuers naechste Intervall
      
      F2_mess_ereignisse = F2_end_ereignis - F2_start_ereignis;  // Impuls-Differenz
      F2_start_ereignis = F2_end_ereignis;      // fuers naechste Intervall

      F2_mess_status = MESSEN;                  // neu starten
      F2_mess_dauer = 0;                        // und Messzeit abwarten
      if(F2_messwert_vorhanden) {               // Ergebnis berechnen und anzeigen/ausgeben

// unkorrigierter Takt (SystemCoreClock) zum Vergleich mit 1pps-Signal
        if(gps_aktiv && F2_mess_ereignisse == 1 &&        // nur 1 Eingangsimpuls messen
            F2_mess_zeit < F_ref*PLUS_5PPM &&   // nur geringe Abweichungen zulassen
            F2_mess_zeit > F_ref*MINUS_5PPM) {  // +/- 5 ppm sind zulssig    
          if(!gps_laeuft) {
            loesche_gps_filter();               // gps war noch nicht aktiviert
            gps_laeuft = 1;                     // ist jetzt initialisiert
          }  
          if(gps_totzeit) {                     // Einschwingen abwarten
            gps_totzeit--;
          } else {
            bewerte_gps_signal(F2_mess_zeit);   // nur engtoleriertes Signal verwenden
          }  
        } else {
          gps_laeuft = 0;                       // kein gueltiges Signal
        }  
        
        F2_ref_frequenz = F1_taktfrequenz;      // zum Berechnen immer aktuellen Wert verwenden
        eff_zeit2 = (double)(F2_mess_zeit ) / F2_ref_frequenz;  // Zeit in Sekunden
        F2_ergebnis = ((double)F2_mess_ereignisse) / eff_zeit2; // Frequenz berechnen
        eff_stellen = F2_stellen;
        if(!eff_stellen) eff_stellen = berechne_stellen2(eff_zeit2);
        Fx_str_mittig("",F2_ergebnis, WANDEL_F, s, eff_stellen);
        lcd_zeile3("  1-pps Signal  "); 
        lcd_messwert_zeile4(s); 
        wandel_Fx("",F2_ergebnis, WANDEL_F, s, eff_stellen);
        if(ser_ausgabe_kanal == F2) sende_str(s);
      } else 
        F2_messwert_vorhanden = 1;              // sperre wieder aufheben
    }
}  

/* direkte Messung mit internen Timern mit max. 10 Stellen/s
Timer2 zhlt die externen Impulse
Timer17 mit die Zeit mit internem Takt
*/
#ifndef AUTO_PLL_FREQUENZ
// einfache Auswertung mit konstanter Referenzfrequenz
void F1_messung(uint8_t init_timer)
{
static uint8_t  init = 0, alte_funktion = 255;
uint8_t eff_stellen;
char p_str[TEMP_STR_LEN];
static char s[TEMP_STR_LEN];

  if(init_timer) { 
    init = 0;                                   // Anzeige + Messung initialisieren
    init_t6();                                  // T6 fr Messdauer in 1 ms Schritten
    init_t7();                                  // F1_TRIGGER aktivieren, max. Messrate vorgeben
    init_t2();                                  // T2 als Ereignisszhler mit capture-modus starten
    init_t17();                                 // T17 als Referenz-Zhler
  }
//  while(1) {    
    if(!init) {
      F1_mess_status = MESSEN;                  // mit MESSEN beginnen
      F1_mess_dauer =  F1_messzeit;             // und 1. Messung sofort abschlieen
      F1_messwert_vorhanden  = 0;               // dabei 1. Ergebnis verwerfen
      init = 1;
      update_header = 1;
    }
 
  if(!ausgabe_sperre && (anzeige_funktion != alte_funktion || update_header)) {  
    update_header = 0;
    alte_funktion = anzeige_funktion;
    loesche_lcd_zeile12();
    switch(anzeige_funktion) {
      case F_UND_P: // 12345678901234567890
        lcd_zeile1("   Signal F1?   ");
        lcd_zeile2("   Signal P1?   ");
        break;
      case FREQUENZ:
        lcd_zeile1("  Frequenz F1   ");
        lcd_zeile2("    Signal?     ");
        break;
      case PERIODE:
        lcd_zeile1("   Periode F1   ");
        lcd_zeile2("    Signal?     ");
        break;
    case DREHZAHL:
        lcd_zeile1("  Drehzahl F1   ");
        lcd_zeile2("    Signal?     ");
        break;
    }
  }
 
    if(F1_mess_dauer >= F1_timeout) {
      set_pin(GPIOA, F1_TRIGGER, 0);            // Trigger-Ausgang wieder auf '0'
      init = 0;                                 // neu initialisieren
    }  
 
    if(F1_mess_status == FERTIG) {              // Regressionsberechnung fertig
      if(F1_messwert_vorhanden) { 
        eff_stellen = F1_stellen;
        if(!eff_stellen) eff_stellen = berechne_stellen1(eff_zeit1, reg_ereignisse);
        if(F1_vorteiler_aktiv)
          F1_ergebnis *= (double)F1_faktor;      // und skalieren

        led_cnt = led_ein_zeit;
        if(ser_ausgabe_kanal == P1) {
          wandel_Fx("",F1_ergebnis, WANDEL_P, s, eff_stellen);
          sende_str(s);
        }
        if(ser_ausgabe_kanal == UPM1) {
          wandel_Fx("",F1_ergebnis*60.0/F1_upm_divisor, WANDEL_D, s, eff_stellen);
          sende_str(s);
        }
        wandel_Fx("",F1_ergebnis, WANDEL_F, s, eff_stellen);
        if(ser_ausgabe_kanal == F1) sende_str(s);
        
        switch(anzeige_funktion) {
          case F_UND_P:
            Fx_str_mittig("",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            messwert_lcd1(p_str);
            Fx_str_mittig("",F1_ergebnis, WANDEL_P, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case FREQUENZ:
            Fx_str_mittig("",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case PERIODE:
            Fx_str_mittig("",F1_ergebnis, WANDEL_P, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case DREHZAHL:
            Fx_str_mittig("",F1_ergebnis*60.0/F1_upm_divisor, WANDEL_D, s, eff_stellen);
            messwert_lcd2(s);
            break;
        }
      } else F1_messwert_vorhanden  = 1;  // sperre wieder aufheben
      F1_mess_status  = MESSEN;           // neue Auswertung zulassen
    }
//  }
}  

#else
// Umschaltung der Referenzfrequenz um geradzahlige, phasenstarre 
// Teilerverhltnisse Fref/Fin ab Fin >= 10 kHZ zu vermeiden
// F1_pll_ref_low   typ. 166,7...  MHz
// F1_pll_ref_mid   typ. 170,0...  MHz
// F1_pll_ref_high  typ. 171,7...  MHz

void F1_messung(uint8_t init_timer)
{
static uint8_t  init = 0, alte_funktion = 255;
uint8_t eff_stellen;
static char s[TEMP_STR_LEN];


  if(init_timer) { 
    berechne_pllm_plln(&F1_pll_m, & F1_pll_n_mid, hse_value, soll_frequenz);
    F1_pll_n_low = F1_pll_n_mid - 2;            // untere Frequenz 3,3 MHz kleiner whlen
    F1_pll_n_high = F1_pll_n_mid + 1;           // obere Frequenz 1,7 MHz grsser whlen
    F1_ref_low = (double)hse_value * F1_pll_n_low / F1_pll_m / PLL_R;     // unterer Wert
    F1_ref_mid = (double)hse_value * F1_pll_n_mid / F1_pll_m / PLL_R;     // mittlerer Wert
    F1_ref_high = (double)hse_value * (F1_pll_n_high) / F1_pll_m / PLL_R; // oberer Wert
    aktiviere_HSE_pll(F1_pll_m, F1_pll_n_low, hse_value);
    F1_ref_akt = F1_ref_low;                    // zunchst unteren Wert verwenden
    berechne_F1_takt();                         // F_ref und F1_offset zu F1_taktfrequenz verrechnen
    init = 0;                                   // Anzeige + Messung initialisieren
    init_t6();                                  // T6 fr Messdauer in 1 ms Schritten
    init_t7();                                  // F1_TRIGGER aktivieren, max. Messrate vorgeben
    init_t2();                                  // T2 als Ereignisszhler mit capture-modus starten
    init_t17();                                 // T17 als Referenz-Zhler
  }
//  while(1) {    
    if(!init) {
      F1_mess_status = MESSEN;                  // mit MESSEN beginnen
      F1_mess_dauer =  F1_messzeit;             // und 1. Messung sofort abschlieen
      F1_messwert_vorhanden  = 0;               // dabei 1. Ergebnis verwerfen
      init = 1;
      update_header = 1;
      F1_pll_status = OFF;                      // zunchst nicht optimieren
      F1_opt_hys = 0;                           // und ohne Hysterese      
    }
 
  if(!ausgabe_sperre && (anzeige_funktion != alte_funktion || update_header)) {  
    update_header = 0;
    alte_funktion = anzeige_funktion;
    loesche_lcd_zeile12();
    switch(anzeige_funktion) {
      case F_UND_P: // 12345678901234567890
        lcd_zeile1("   Signal F1?   ");
        lcd_zeile2("   Signal P1?   ");
        break;
      case FREQUENZ:
        lcd_zeile1("  Frequenz F1   ");
        lcd_zeile2("    Signal?     ");
        break;
      case PERIODE:
        lcd_zeile1("   Periode F1   ");
        lcd_zeile2("    Signal?     ");
        break;
    case DREHZAHL:
        lcd_zeile1("  Drehzahl F1   ");
        lcd_zeile2("    Signal?     ");
        break;
    }
  }
 
    if(F1_mess_dauer >= F1_timeout) {
      set_pin(GPIOA, F1_TRIGGER, 0);            // Trigger-Ausgang wieder auf '0'
      init = 0;                                 // neu initialisieren
    }  
 
    if(F1_mess_status == FERTIG) {              // Regressionsberechnung fertig
      if(F1_messwert_vorhanden) { 
        switch(F1_reg_modus) {
          case OPTIMIERT:
            if(F1_ergebnis >= (F1_OPTIMIERUNG-F1_opt_hys)) {
              F1_opt_hys = F1_OPTIMIERUNGS_HYS;     // Abschaltschwelle bei 80%

              F_temp = (F1_ref_low / F1_ergebnis);
              F1_rest_low = (int)F_temp;
              F1_rest_low = abs((int)((F_temp - F1_rest_low) * REST_SKALIERUNG)-REST_SKALIERUNG/2);
              F_temp = (F1_ref_mid / F1_ergebnis);
              F1_rest_mid = (int)F_temp;
              F1_rest_mid = abs((int)((F_temp - F1_rest_mid) * REST_SKALIERUNG)-REST_SKALIERUNG/2);
              F_temp = (F1_ref_high / F1_ergebnis);
              F1_rest_high = (int)F_temp;
              F1_rest_high = abs((int)((F_temp - F1_rest_high) * REST_SKALIERUNG)-REST_SKALIERUNG/2);

              if(F1_rest_high <= F1_rest_low) {     // Teilerrest soll mglichst nahe 0,5 sein
                if(F1_rest_mid <= F1_rest_high)
                   F1_pll_status = MID;
                else
                  F1_pll_status = HIGH;
              } else {
                if(F1_rest_mid <= F1_rest_low)
                   F1_pll_status = MID;
                else
                  F1_pll_status = LOW;
              }
              if(F1_pll_status == HIGH) {     
                F_temp = F1_ref_high;           // hhere Frequenz ist besser geeignet
                F1_pll_n_temp = F1_pll_n_high;  // passender Teiler dazu
              } else if(F1_pll_status == MID) {     
                F_temp = F1_ref_mid;            // tiefere Frequenz ist besser geeignet
                F1_pll_n_temp = F1_pll_n_mid;   // passender Teiler dazu
              } else {
                F_temp = F1_ref_low;            // tiefere Frequenz ist besser geeignet
                F1_pll_n_temp = F1_pll_n_low;   // passender Teiler dazu
              }  
            } else {                            // Eingangsfrequenz zu klein
              F1_pll_n_temp = F1_pll_n_high;    // dann hohe Frequenz als default
              F_temp = F1_ref_high;
              F1_pll_status = OFF;
              F1_opt_hys = 0;
            }  
            break;
          case HOCH_LF:  
             F_temp = F1_ref_low;               // tiefere Frequenz ist besser geeignet
             F1_pll_n_temp = F1_pll_n_low;      // passender Teiler dazu
             F1_pll_status = OFF;
            break;
          case HOCH_HF:  
             F1_pll_n_temp = F1_pll_n_high;     // passender Teiler dazu
             F_temp = F1_ref_high;              // hhere Frequenz ist besser geeignet
             F1_pll_status = OFF;
             break;
          case HOCH_MF:  
          default:  
             F1_pll_n_temp = F1_pll_n_mid;     // passender Teiler dazu
             F_temp = F1_ref_mid;              // hhere Frequenz ist besser geeignet
             F1_pll_status = OFF;
             break;
        } // switch(F1_reg_modus)

        if(F_temp != F1_ref_akt) {              // nderung notwendig: PLL wird neu eingestellt und nchste Messung verworfen
            F1_ref_akt = F_temp;                // und merken
            aktiviere_HSE_pll(F1_pll_m, F1_pll_n_temp, hse_value);  // neue Einstellung
            berechne_F1_takt();                 // F_ref und F1_offset zu F1_taktfrequenz verrechnen
// Timing auch anpassen
            TIM6->PSC = SystemCoreClock/MHZ_FAKTOR-1; // Vorteiler auf 1 MHz
            TIM7->ARR = SystemCoreClock/F_SAMPLES-1;  // max. Abfragefrequenz
// und Neustart einer dummy-Messung zum Einschwingen der PLL            
            F1_messwert_vorhanden = 0;          // nchste Messung ist ungltig
            F1_mess_dauer =  F1_messzeit-3;     // und Messung nach 1-2 ms abschlieen
            F2_messwert_vorhanden = 0;          // F2-Ergebnis kippen
            set_baudrate2(baud_tab[baudraten_index]); // da SystemCoreClock verndert wurde
            F1_mess_status = MESSEN;            // mit MESSEN beginnen
        }
// Ergebnis ist bewertet, nun noch anzeigen/ausgeben        
        eff_stellen = F1_stellen;
        if(!eff_stellen) eff_stellen = berechne_stellen1(eff_zeit1, reg_ereignisse);
        if(F1_vorteiler_aktiv)
          F1_ergebnis *= (double)F1_faktor;     // und skalieren

        led_cnt = led_ein_zeit;
        if(ser_ausgabe_kanal == P1) {
          wandel_Fx("",F1_ergebnis, WANDEL_P, s, eff_stellen);
          sende_str(s);
        }
        if(ser_ausgabe_kanal == UPM1) {
          wandel_Fx("",F1_ergebnis*60.0/F1_upm_divisor, WANDEL_D, s, eff_stellen);
          sende_str(s);
        }
        wandel_Fx("",F1_ergebnis, WANDEL_F, s, eff_stellen);
        if(ser_ausgabe_kanal == F1) sende_str(s);
        
        switch(anzeige_funktion) {
          case F_UND_P:
            Fx_str_mittig("",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            messwert_lcd1(p_str);
            Fx_str_mittig("",F1_ergebnis, WANDEL_P, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case FREQUENZ:
#ifdef ZEIGE_BEWERTUNG            
            if(F1_pll_alt == HIGH && F1_reg_modus == OPT)      // testweise '+' voranstellen
              Fx_str_mittig("+",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            else if(F1_pll_alt == LOW && F1_reg_modus == OPT)  // testweise '-' voranstellen
              Fx_str_mittig("-",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            else if(F1_pll_alt == MID && F1_reg_modus == OPT)  // testweise '=' voranstellen
              Fx_str_mittig("=",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            else
#endif              
              Fx_str_mittig("",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case PERIODE:
            Fx_str_mittig("",F1_ergebnis, WANDEL_P, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case DREHZAHL:
            Fx_str_mittig("",F1_ergebnis*60.0/F1_upm_divisor, WANDEL_D, s, eff_stellen);
            messwert_lcd2(s);
            break;
        }
        F1_pll_alt = F1_pll_status;       // neue Bewertung verzgert anzeigen
      } else F1_messwert_vorhanden  = 1;  // sperre wieder aufheben
      F1_mess_status  = MESSEN;           // neue Auswertung zulassen
    }
//  }
}  
#endif