#define G431_LCD

// Anschlu einer LC-Anzeige 2 x 16 -> 4 x 20
// lcd_init() muss als erste Funktion aufgerufen werden. Danach sollten
// lcd_cmd() und lcd_zeichen() Befehle und Daten korrekt ausgeben.

/*
Das LCD-Modul wird im 4-Bit Modus betrieben. D0-D3 liegen daher auf GND.
Verwendet werden von PortA die Pins 4-11 wie folgt:

  LCD-Pin   Funktion        STM32G431

    1         GND            GND-Pin
    2         +5V       +5V Versorgungsspannung
    3       Kontrast    per DAC_1 PA4
    4         RS              PA5
    5         R/W             PA6
    6         E               PA7
    7       D0(GND)
    8       D1(GND)
    9       D2(GND)
    10      D3(GND)
    11        D4              PA8
    12        D5              PA9
    13        D6              PA10
    14        D7              PA11


2021-01-09
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/


#include "system_G431.h"

#define LCD_INIT_DELAY    50000     // 50 ms
#define LCD_NIBBLE_DELAY  10000     // 10 ms
uint8_t lcd_vorhanden=1;            // default

#define ZEILE1      0x7f            // jeweils 1 addieren fuer 1. Spalte
#define ZEILE2      0xbf            // dto.
#define BUSY_FLAG   0x80            // Bit7 vom LCD-Status
  
uint8_t zeilen_tab[] = {0,ZEILE1,ZEILE2,ZEILE1,ZEILE2}; /* alle werte -1 */
uint8_t lcd_zeile_start[5];

uint8_t lcd_breite = DEF_LCD_BREITE;
uint8_t lcd_kontrast = DEF_LCD_KONTRAST;
uint8_t lcd_busy;

void lcd_impuls(); 


// 16 oder 20 Zeichen/Zeile initialisieren
void init_zeilen_breite(uint8_t breite)
{
int i, offset=1;
  if(breite != MIN_LCD_BREITE) offset += (breite-MIN_LCD_BREITE)/2;       // 2 Stellen weiter nach rechts schreiben
  lcd_breite = breite;
  zeilen_tab[3] = ZEILE1 + breite;            // fr Zeile 3
  zeilen_tab[4] = ZEILE2 + breite;            // fr Zeile 4
  for(i=0; i<5; i++)
    lcd_zeile_start[i] = zeilen_tab[i]+offset;
} 



// LCD kontrast per DAC1
void setze_kontrast(uint8_t wert)
{
static uint8_t init;
  if(!init) {
    RCC->AHB2ENR |= RCC_AHB2ENR_DAC1EN;
    init = 1;
    DAC->CR = DAC_CR_EN1;
    DAC->MCR = DAC_MCR_HFSEL_1;
  }
  if(wert > MAX_LCD_KONTRAST)             // Max. Wert begrenzen
    wert = DEF_LCD_KONTRAST;
  DAC->DHR8R1 = wert;
}  
  
uint8_t lcd_lesen(void)                   // oberstes Bit ist busy-flag, Rest: Adresse
{
  uint8_t temp = 0;
// Datenleitungen auf Eingang
  LCD_PORT->MODER &= ~(CLR_MODER<<LCD_D4*2 | 
                    CLR_MODER<<LCD_D5*2 | 
                    CLR_MODER<<LCD_D6*2 | 
                    CLR_MODER<<LCD_D7*2);
  
  set_pin(LCD_PORT, LCD_READ, 1);         // Lesen aktivieren
  delay_us(1);
  set_pin(LCD_PORT, LCD_STROBE, 1);       // E zum Lesen setzen
  delay_us(1);
  temp = (LCD_PORT->IDR >> LCD_DATA_OFFSET) & 0xf;  // oberes nibble zuerst
  temp <<= 4;
  set_pin(LCD_PORT, LCD_STROBE, 0);       // und wieder auf 0
  delay_us(1);
  set_pin(LCD_PORT, LCD_STROBE, 1);       // nchstes nibble lesen
  delay_us(1);
  temp |= (LCD_PORT->IDR >> LCD_DATA_OFFSET) & 0xf; // unteres nibble zuletzt
  set_pin(LCD_PORT, LCD_STROBE, 0);       // und wieder auf 0
  set_pin(LCD_PORT, LCD_READ, 0);         // dto
  delay_us(1);
  
// Datenleitungen wieder auf Ausgang  
  LCD_PORT->MODER |= (OUT<<LCD_D4*2 | 
                   OUT<<LCD_D5*2 | 
                   OUT<<LCD_D6*2 | 
                   OUT<<LCD_D7*2);
  delay_us(1);
  return(temp);                           // busy-flag + Adresse
}

// Status lesen; busy-flag == 0, wenn neue Ausgabe mglich
uint8_t lcd_status(void)                  // oberstes Bit ist busy-flag, Rest: Adresse
{
  set_pin(LCD_PORT, LCD_CMD, 0);          // CMD ausfhren
  delay_us(1);
  return(lcd_lesen());
} 


void lcd_impuls(void)                     // Schreibimpuls mit Mindestlaenge
{
  delay_us(1);
  set_pin(LCD_PORT, LCD_STROBE, 1);       // 1 = aktiv
  delay_us(1);
  set_pin(LCD_PORT, LCD_STROBE, 0);       // und wieder auf 0
  delay_us(1);
}

void lcd_nibble(int8_t c)                 // 4-bit-Wert an PA8-PA11 ausgeben
{
  c >>= 4;
  c &= 0xf;
  LCD_PORT->BSRR = ((~c << LCD_DATA_OFFSET) << 16); // alle Datenleitungen auf 0
  LCD_PORT->BSRR = c << LCD_DATA_OFFSET;            // danach aktive Bits setzen
  lcd_impuls();
}

void lcd_out(int8_t z, int8_t mode)     // Ausgabe von Daten (mode==1) oder CMDs (mode==0)
{
  lcd_busy = 1;
    while(lcd_status() & BUSY_FLAG);    // vor neuer Ausgabe busy-flag abwarten
    if(mode) set_pin(LCD_PORT, LCD_CMD, 1);
    else set_pin(LCD_PORT, LCD_CMD, 0);
    lcd_nibble(z);                      // obere Bits zuerst
    lcd_nibble(z << 4);                 // untere Bits

  lcd_busy = 0;
}

uint8_t lcd_test(void)               
{
uint8_t temp = 0;
  lcd_busy = 1;
  set_pin(LCD_PORT, LCD_CMD, 1);  // immer setzen, um Daten zu lesen
  delay_us(1);
  temp = lcd_lesen();
  lcd_busy = 0;
  return(temp);
}  

void lcd_cmd(uint8_t z)           // einen LCD-Befehl ausgeben
{
  lcd_out(z,0);
}

void lcd_zeichen(char z)          // ein Zeichen ausgeben
{
  lcd_out(z,1);
}

void lcd_string(char *s)          // Zeichenkette auf LCD
{   
int8_t temp;
  if(lcd_vorhanden) {
    while((temp=*s++) != 0) {
      lcd_zeichen(temp);
    }
  }
}

void lcd_init_nibble(int8_t c)      // 4-bit-Wert ausgeben LCD_INIT
{
  lcd_busy = 1;
  lcd_nibble(c);
  lcd_busy = 0;
  delay_us(LCD_NIBBLE_DELAY);       // jeweils 10 ms
}

void lcd_zeile1(char *s)            // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    lcd_cmd(lcd_zeile_start[1]);    // 1.Zeile, 1.Spalte
    lcd_string(s);
  }
}
void lcd_zeile2(char *s)            // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    lcd_cmd(lcd_zeile_start[2]);    // 2.Zeile, 1.Spalte
    lcd_string(s);
  }
}

void lcd_zeile3(char *s)            // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    lcd_cmd(lcd_zeile_start[3]);    // 3.Zeile, 1.Spalte
    lcd_string(s);
  }
}
void lcd_zeile4(char *s)            // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    lcd_cmd(lcd_zeile_start[4]);    // 4.Zeile, 1.Spalte
    lcd_string(s);
  }
}


void lcd_cursor_on()
{
  lcd_cmd(0xf);
}

void lcd_cursor_off()
{
  lcd_cmd(0xc);
}

void lcd_zeile_spalte(uint8_t zeile, uint8_t spalte)  // Schreibposition setzen
{
  if(zeile && spalte) lcd_cmd(zeilen_tab[zeile] + spalte);
}

void loesche_lcd_zeile(uint8_t zeile, uint8_t n)
{
uint8_t i;  
  if(lcd_vorhanden) {
    lcd_cmd(zeilen_tab[zeile] + 1);         // Startposition
    for(i=0; i<n; i++) lcd_zeichen(' ');    // Leerzeichen schreiben
  }  
}  

void lcd_clear()
{
  lcd_cursor_off();
  lcd_cmd(1);
}

void loesche_lcd_zeile1()
{
  loesche_lcd_zeile(1, lcd_breite);
}
void loesche_lcd_zeile2()
{
  loesche_lcd_zeile(2, lcd_breite);
}
void loesche_lcd_zeile12()
{
  loesche_lcd_zeile(1, lcd_breite);
  loesche_lcd_zeile(2, lcd_breite);
}

void loesche_lcd_zeile34()
{
  loesche_lcd_zeile(3, lcd_breite);
  loesche_lcd_zeile(4, lcd_breite);
}

void lcd_messwert_zeile1(char *s)         // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    if(lcd_breite == DEF_LCD_BREITE)
      lcd_cmd(lcd_zeile_start[1]);        // 1.Zeile, 1.Spalte
    else
      lcd_cmd(lcd_zeile_start[1]-2);      // 1.Zeile, 1.Spalte
    lcd_string(s);
  }
}

void lcd_messwert_zeile2(char *s)         // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    if(lcd_breite == DEF_LCD_BREITE)
      lcd_cmd(lcd_zeile_start[2]);        // 2.Zeile, 1.Spalte
    else
      lcd_cmd(lcd_zeile_start[2]-2);      // 2.Zeile, 1.Spalte
    lcd_string(s);
  }
}

void lcd_messwert_zeile3(char *s)         // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    if(lcd_breite == DEF_LCD_BREITE)
      lcd_cmd(lcd_zeile_start[3]);        // 3.Zeile, 1.Spalte
    else
      lcd_cmd(lcd_zeile_start[3]-2);      // 3.Zeile, 1.Spalte
    lcd_string(s);
  }
}

void lcd_messwert_zeile4(char *s)         // komplette Zeile ausgeben
{
  if(lcd_vorhanden) {
    if(lcd_breite == DEF_LCD_BREITE)
      lcd_cmd(lcd_zeile_start[4]);        // 4.Zeile, 1.Spalte
    else
      lcd_cmd(lcd_zeile_start[4]-2);      // 4.Zeile, 1.Spalte
    lcd_string(s);
  }
}

  
uint8_t lcd_init(void)        // Port A4 - Port A11 werden gebraucht
{
uint32_t temp;
	RCC->AHB2ENR |= RCC_AHB2ENR_GPIOAEN;        // PortA aktivieren

// Ausgnge von PortA push-pull output*/
  temp = LCD_PORT->MODER &
              ~(CLR_MODER << LCD_CMD*2 | 
                CLR_MODER << LCD_READ*2 | 
                CLR_MODER << LCD_STROBE*2 |
                CLR_MODER << LCD_D4*2 | 
                CLR_MODER << LCD_D5*2 | 
                CLR_MODER << LCD_D6*2 | 
                CLR_MODER << LCD_D7*2);
  LCD_PORT->MODER = temp |
               (OUT << LCD_CMD*2 | 
                OUT << LCD_READ*2 | 
                OUT << LCD_STROBE*2 | 
                OUT << LCD_D4*2 | 
                OUT << LCD_D5*2 | 
                OUT << LCD_D6*2 | 
                OUT << LCD_D7*2);
  LCD_PORT->PUPDR |= 
                (PULL_UP << LCD_D4*2 |    // fr Statusabfrage
                 PULL_UP << LCD_D5*2 | 
                 PULL_UP << LCD_D6*2 | 
                 PULL_DOWN << LCD_D7*2);  // ohne LCD liefert busy-Abfrage "bereit"
/*  
  LCD_PORT->OSPEEDR |= 
               (HIGH_SPEED << LCD_CMD*2 | 
                HIGH_SPEED << LCD_READ*2 | 
                HIGH_SPEED << LCD_STROBE*2 | 
                HIGH_SPEED << LCD_D4*2 | 
                HIGH_SPEED << LCD_D5*2 | 
                HIGH_SPEED << LCD_D6*2 | 
                HIGH_SPEED << LCD_D7*2);
*/
  init_zeilen_breite(lcd_breite);

  delay_us(LCD_INIT_DELAY);   // 50 ms nach Reset

  lcd_init_nibble(0x30);      // 8 Bit Modus
  lcd_init_nibble(0x30);      // wiederholen
  lcd_init_nibble(0x30);      // wiederholen
  lcd_init_nibble(0x20);      // dann auf 4 Bit Modus
  lcd_cmd(0x28);              // 2 oder 4 Zeilen
  lcd_cmd(0x28);              // wiederholen

  lcd_cmd(0x8);               // LCD off            
  lcd_cmd(0x6);               // LCD auto. increment
  lcd_cmd(0x17);              // LCD char. mode; PWR on
  lcd_cmd(0x1);               // LCD clear
  lcd_cmd(0x2);               // LCD home
  
  lcd_cmd(0xc);               // LCD on; ohne Cusor und Blinken
  lcd_cmd(1);                 // LCD clear
  setze_kontrast(lcd_kontrast);
  delay_us(LCD_INIT_DELAY);   // 50 ms nach Init
  
  return(lcd_status());
}

